@@ -27,13 +27,14 @@
 | Applied Coherent patch WRITETEXT                 | J.L. Bezemer     |290312|
 | Added Francois Perrad LINT patches               | Francois Perrad  |221016|
 | Fixed bug in JSLEEP entry of Interpret()         | J.L. Bezener     |140920|
+| Fixed bug in AddTask()                           | J.L. Bezener     |140322|
 +--------------------------------------------------+------------------+------+
 |REMARKS                                           |    Programmer    | Date |
 +--------------------------------------------------+------------------+------+
 | Demonstration for 4th toolkit                    | J.L. Bezemer     |010207|
 +--------------------------------------------------+------------------+------+
 
-  Copyright (C) 2007, 2020 J.L. Bezemer
+  Copyright (C) 2007, 2022 J.L. Bezemer
 
   This file is part of 4tH
 
@@ -196,14 +197,14 @@ This function converts the pid to a job. It returns ToS if not found.
 */
 
 #ifndef ARCHAIC
-  int ToJob (int pid) 
+  int ToJob (int pid)
 #else
   int ToJob (pid) int pid;
 #endif
 
 {
   int job = INIT;
-                                       /* try to find a job with this pid */ 
+                                       /* try to find a job with this pid */
   while ((job < ToS) && (Process [job].Id != pid)) job++;
   return (job);
 }
@@ -286,17 +287,22 @@ if successful.
 {
   static int TaskId = 0;
 
-  if ((Prog) && (ToS < MAXJOBS))    /* if empty slot found */
-     {                              /* add the job name */
-       Process [ToS].Name [MAXNAME - 1] = '\0';
-       strncpy (Process [ToS].Name, Name, MAXNAME - 1);
-       Process [ToS].Job = Prog;    /* add the task */
-       Process [ToS].Stat = Mode;   /* set the status */
-       Process [ToS].Id = (TaskId++ & INT_MAX);  
-       printf ("Task %d started\n", Process [ToS].Id);
-       return (ToS++);              /* increment top of stack */
-     }
-  return (ToS);                     /* return always top of stack */
+  if (Prog)                            /* do we have a program */
+    {
+      if (ToS < MAXJOBS)               /* if empty slot found */
+         {                             /* add the job name */
+           strncpy (Process [ToS].Name, Name, MAXNAME - 1);
+           Process [ToS].Name [MAXNAME - 1] = '\0';
+           Process [ToS].Job = Prog;   /* add the task */
+           Process [ToS].Stat = Mode;  /* set the status */
+           Process [ToS].Id = (TaskId++ & INT_MAX);
+           printf ("Task %d started\n", Process [ToS].Id);
+           return (ToS++);             /* increment top of stack */
+         }
+      else free_4th (Prog);            /* no more slots, so kill it */
+    }
+
+  return (ToS);                        /* return always top of stack */
 }
 
 
@@ -461,7 +467,7 @@ applet. It returns a job.
     {                                  /* if still current job and awake */
       stk = exec_4th (Process [job].Job, argn, args, 0);
       fflush (stdout);                 /* get the user stack address */
-      if (Process [job].Job->Offset == 0) 
+      if (Process [job].Job->Offset == 0)
          {                             /* remove monitor when terminated */
            KillTask (Process [job].Id); return (++job);
          }                             /* return next job */
@@ -543,3 +549,4 @@ starts the multitasker. Finally, it returns success.
                                        /* start the multitasker */
   return (EXIT_SUCCESS);               /* return success */
 }
+
